import { readExternalEntries } from './config/imports/external.js';
import { pathsForImport } from './config/imports/paths.js';
import type { ImportKind, RawEntry } from './config-schema.js';
export { pathsForImport, readExternalEntries };
export declare function toFileUrl(filePath: string): URL;
export type { ImportKind, RawEntry };
//# sourceMappingURL=config-imports.d.ts.map