import type { CommandInput } from './types.js';
export interface GenerateFlags {
    server?: string;
    name?: string;
    command?: CommandInput;
    description?: string;
    output?: string;
    bundler?: 'rolldown' | 'bun';
    bundle?: boolean | string;
    compile?: boolean | string;
    runtime?: 'node' | 'bun';
    timeout: number;
    minify?: boolean;
    from?: string;
    dryRun: boolean;
    includeTools?: string[];
    excludeTools?: string[];
}
export declare function parseGenerateFlags(args: string[]): GenerateFlags;
//# sourceMappingURL=flags.d.ts.map