interface ParsedCallExpression {
    server?: string;
    tool: string;
    args: Record<string, unknown>;
    positionalArgs?: unknown[];
}
export declare function parseCallExpressionFragment(raw: string): ParsedCallExpression | null;
export {};
//# sourceMappingURL=call-expression-parser.d.ts.map