export interface DaemonLaunchOptions {
    readonly configPath: string;
    readonly configExplicit?: boolean;
    readonly rootDir?: string;
    readonly socketPath: string;
    readonly metadataPath: string;
    readonly extraArgs?: string[];
}
export declare function launchDaemonDetached(options: DaemonLaunchOptions): void;
//# sourceMappingURL=launch.d.ts.map