export declare const DEBUG_HANG: boolean;
export declare function dumpActiveHandles(label: string): void;
export declare function terminateChildProcesses(label: string): void;
//# sourceMappingURL=runtime-debug.d.ts.map