export type DaemonRequestMethod = 'callTool' | 'listTools' | 'listResources' | 'closeServer' | 'status' | 'stop';
export interface DaemonRequest<T extends DaemonRequestMethod = DaemonRequestMethod, P = unknown> {
    readonly id: string;
    readonly method: T;
    readonly params: P;
}
export interface DaemonResponse<T = unknown> {
    readonly id: string;
    readonly ok: boolean;
    readonly result?: T;
    readonly error?: {
        readonly message: string;
        readonly code?: string;
    };
}
export interface CallToolParams {
    readonly server: string;
    readonly tool: string;
    readonly args?: Record<string, unknown>;
    readonly timeoutMs?: number;
}
export interface ListToolsParams {
    readonly server: string;
    readonly includeSchema?: boolean;
    readonly autoAuthorize?: boolean;
}
export interface ListResourcesParams {
    readonly server: string;
    readonly params?: Record<string, unknown>;
}
export interface CloseServerParams {
    readonly server: string;
}
export interface StatusResult {
    readonly pid: number;
    readonly startedAt: number;
    readonly configPath: string;
    readonly configMtimeMs?: number | null;
    readonly configLayers?: Array<{
        readonly path: string;
        readonly mtimeMs: number | null;
    }>;
    readonly socketPath: string;
    readonly logPath?: string;
    readonly servers: Array<{
        readonly name: string;
        readonly connected: boolean;
        readonly lastUsedAt?: number;
    }>;
}
//# sourceMappingURL=protocol.d.ts.map