import type { ServerDefinition } from './config.js';
import type { Logger } from './logging.js';
export declare function maybeEnableOAuth(definition: ServerDefinition, logger: Logger): ServerDefinition | undefined;
export declare function isUnauthorizedError(error: unknown): boolean;
//# sourceMappingURL=runtime-oauth-support.d.ts.map