import type { ServerToolInfo } from './runtime.js';
export interface GenerateCliOptions {
    readonly serverRef: string;
    readonly configPath?: string;
    readonly rootDir?: string;
    readonly outputPath?: string;
    readonly runtime?: 'node' | 'bun';
    readonly bundler?: 'rolldown' | 'bun';
    readonly bundle?: boolean | string;
    readonly timeoutMs?: number;
    readonly minify?: boolean;
    readonly compile?: boolean | string;
    readonly includeTools?: string[];
    readonly excludeTools?: string[];
}
export declare function generateCli(options: GenerateCliOptions): Promise<{
    outputPath: string;
    bundlePath?: string;
    compilePath?: string;
}>;
declare function applyToolFilters(tools: ServerToolInfo[], includeTools?: string[], excludeTools?: string[]): ServerToolInfo[];
export declare const __test: {
    applyToolFilters: typeof applyToolFilters;
    getEnumValues: typeof import("./cli/generate/tools.js").getEnumValues;
    getDescriptorDefault: typeof import("./cli/generate/tools.js").getDescriptorDefault;
    buildPlaceholder: typeof import("./cli/generate/tools.js").buildPlaceholder;
    buildExampleValue: typeof import("./cli/generate/tools.js").buildExampleValue;
    pickExampleLiteral: typeof import("./cli/generate/tools.js").pickExampleLiteral;
    buildFallbackLiteral: typeof import("./cli/generate/tools.js").buildFallbackLiteral;
};
export {};
//# sourceMappingURL=generate-cli.d.ts.map