export declare function handleCall(runtime: Awaited<ReturnType<typeof import('../runtime.js')['createRuntime']>>, args: string[]): Promise<void>;
export declare function printCallHelp(): void;
//# sourceMappingURL=call-command.d.ts.map