export declare function fileExists(filePath: string): Promise<boolean>;
export declare function parseJsonBuffer(buffer: string): unknown;
export declare function isRecord(value: unknown): value is Record<string, unknown>;
//# sourceMappingURL=shared.d.ts.map