import type { Runtime } from '../runtime.js';
import { type EphemeralServerResolution, type EphemeralServerSpec } from './adhoc-server.js';
interface PrepareEphemeralServerTargetOptions {
    runtime: Runtime;
    target?: string;
    ephemeral?: EphemeralServerSpec;
    nameHints?: string[];
    reuseFromSpec?: boolean;
}
interface PrepareEphemeralServerTargetResult {
    target?: string;
    resolution?: EphemeralServerResolution;
}
export declare function prepareEphemeralServerTarget(options: PrepareEphemeralServerTargetOptions): Promise<PrepareEphemeralServerTargetResult>;
export {};
//# sourceMappingURL=ephemeral-target.d.ts.map